/*
   Author:           Paul Conway
   Contact Details:  PEI Technologies Ltd.
                     Foundation Building
                     National Technological Park
                     Limerick
                     Ireland
                     http://www.ul.ie/~pei

   Release Date:     May 1999.
   Version:          1.01
   Company:          Analog Devices
*/

#include <aduc812.h>
#include "datatype.h"
#include "function.h"
#include "tii.h"
#include "teds.h"
#include "stim.h"


// Module Level Variables:
// ----------------------

// This variable is required by the 'trigger' functions:
//
static unsigned char mucTrigChanAddr = 0xFF;


// Data Transport Functions:
// ------------------------

// Note that there are 256 bytes of Data RAM available on-chip.
// This transport length cannot approach that amount (due to stack 
// and running data-space requirements)
// 
#define MAX_TRANSPORT_LENGTH 100

boolean DAT_TransportActive(void)
{
   return TII_DetectNIOE();
}
//
// The checksum will be the one's complement of the sum (modulo 2^16) of all 
// the data structure's preceding bytes - including the initial length field
// and excluding the checksum field. 
//
// The data structure length field is 4 bytes long. It represents the total
// number of bytes in the data block, excluding itself (i.e. excluding 4 bytes)
//
U16C DAT_CalcChecksum(unsigned char *pBuf)
{
   U32L Offset=0;
   U32L NumBytes=0;
   U16C ChkSum=0;

   NumBytes = (U32L) *(U32L*)pBuf;

      // Include the length field & exclude the checksum field (i.e.4-2 => +2).
      // NumBytes will then represent all the bytes excluding the checksum.
      //
   NumBytes += 2;  

   for(Offset=0; Offset<NumBytes; Offset++)
   {
      ChkSum += *(pBuf+Offset);
   }

      // The actual checksum is the 1's complement => negate bits.
      //
   return ~ChkSum;
}

boolean DAT_ReadMetaTEDS(void)
{
   boolean             bRet=FALSE;
   unsigned char idata pDataTransportBuffer[MAX_TRANSPORT_LENGTH];
       
      // Meta TEDS *must* be implemented... if it's there, send it.
      //
   if(TEDS_GetTEDSHandle(TEDS_META, NUL, pDataTransportBuffer))
   {
      U32L TEDSLength = (U32L) (*(U32L*)pDataTransportBuffer+sizeof(U32L));

      bRet = TII_ReadFrame(TEDSLength, pDataTransportBuffer);
   }
      // ELSE If there has been a problem finding it.. return a null TEDS field
      //
   else
   {
      U32L NullTEDS = ZERO_LENGTH_FIELD;
      bRet = TII_ReadFrame( 4, (unsigned char*) &NullTEDS );
   }

   return bRet;
}

boolean DAT_ReadChannelTEDS(U8E ChanNum)
{
   boolean             bRet=FALSE;
   unsigned char idata pDataTransportBuffer[MAX_TRANSPORT_LENGTH];
       
      // Channel TEDS *must* be implemented... if it's there, send it.
      //
   if(TEDS_GetTEDSHandle(TEDS_CHANNEL, ChanNum, pDataTransportBuffer))
   {
      U32L TEDSLength = (U32L) (*(U32L*)pDataTransportBuffer+sizeof(U32L));

      bRet = TII_ReadFrame(TEDSLength, pDataTransportBuffer);
   }
      // ELSE If there has been a problem finding it (of if the Channel Number
      //      specified is invalid)... return a null TEDS field
      //
   else
   {
      U32L NullTEDS = ZERO_LENGTH_FIELD;
      bRet = TII_ReadFrame( 4, (unsigned char*) &NullTEDS );
   }

   return bRet;
}

boolean DAT_ReadMetaIdTEDS(void)
{
   boolean             bRet=FALSE;
   unsigned char idata pDataTransportBuffer[MAX_TRANSPORT_LENGTH];

      // IF there is a Meta-ID TEDS implemented, read it then send it:
      //
   if(TEDS_GetTEDSHandle(TEDS_META_ID, NUL, pDataTransportBuffer))
   {
      U32L TEDSLength = (U32L) (*(U32L*)pDataTransportBuffer+sizeof(U32L));

      bRet = TII_ReadFrame(TEDSLength, pDataTransportBuffer);
   }
      // ELSE There is no Meta-ID TEDS implemented: return a 'field-
      //      length' (4 bytes long) containing '0' to indicate that there 
      //      will be no data-fields to follow.
   else
   {
      U32L NullTEDS = ZERO_LENGTH_FIELD;
      bRet = TII_ReadFrame( 4, (unsigned char*) &NullTEDS );
   }

   return bRet;
}

boolean DAT_ReadChannelIdTEDS(U8E ChanNum)
{
   boolean             bRet=FALSE;
   unsigned char idata pDataTransportBuffer[MAX_TRANSPORT_LENGTH];

      // IF there is a Channel-ID TEDS implemented, read it then send it:
      //
   if(TEDS_GetTEDSHandle(TEDS_CHANNEL_ID, ChanNum, pDataTransportBuffer))
   {
      U32L TEDSLength = (U32L) (*(U32L*)pDataTransportBuffer+sizeof(U32L));

      bRet = TII_ReadFrame(TEDSLength, pDataTransportBuffer);
   }
      // ELSE There is no Channel-ID TEDS implemented: return a 'field-
      //      length' (4 bytes long) containing '0' to indicate that there 
      //      will be no data-fields to follow.
   else
   {
      U32L NullTEDS = ZERO_LENGTH_FIELD;
      bRet = TII_ReadFrame( 4, (unsigned char*) &NullTEDS );
   }

   return bRet;
}

boolean DAT_ReadCalibrationTEDS(void)
{
   boolean             bRet=FALSE;
   unsigned char idata pDataTransportBuffer[MAX_TRANSPORT_LENGTH];

      // IF there is a Calibration TEDS implemented, read it then send it:
      //
   if(TEDS_GetTEDSHandle(TEDS_CALIBRATION, NUL, pDataTransportBuffer))
   {
      U32L TEDSLength = (U32L) (*(U32L*)pDataTransportBuffer+sizeof(U32L));

      bRet = TII_ReadFrame(TEDSLength, pDataTransportBuffer);
   }
      // ELSE There is no 'Calibration TEDS' implemented: return a 'field-
      //      length' (4 bytes long) containing '0' to indicate that there 
      //      will be no data-fields to follow.
   else
   {
      U32L NullTEDS = ZERO_LENGTH_FIELD;
      bRet = TII_ReadFrame( 4, (unsigned char*) &NullTEDS );
   }

   return bRet;
}

boolean DAT_ReadCalibrationIdTEDS(void)
{
   boolean             bRet=FALSE;
   unsigned char idata pDataTransportBuffer[MAX_TRANSPORT_LENGTH];

      // IF there is a 'Calibration-ID' TEDS implemented, read it then send it:
      //
   if(TEDS_GetTEDSHandle(TEDS_CALIB_ID, NUL, pDataTransportBuffer))
   {
      U32L TEDSLength = (U32L) (*(U32L*)pDataTransportBuffer+sizeof(U32L));

      bRet = TII_ReadFrame(TEDSLength, pDataTransportBuffer);
   }
      // ELSE There is no 'Calibration-ID' TEDS implemented: return a 'field-
      //      length' (4 bytes long) containing '0' to indicate that there 
      //      will be no data-fields to follow.
   else
   {
      U32L NullTEDS = ZERO_LENGTH_FIELD;
      bRet = TII_ReadFrame( 4, (unsigned char*) &NullTEDS );
   }

   return bRet;
}

boolean DAT_ReadEndUserAppTEDS(void)
{
   boolean             bRet=FALSE;
   unsigned char idata pDataTransportBuffer[MAX_TRANSPORT_LENGTH];

      // IF there is an 'End User Application Specific' TEDS implemented, 
      // read it then send it:
      //
   if(TEDS_GetTEDSHandle(TEDS_END_USER, NUL, pDataTransportBuffer))
   {
      U32L TEDSLength = (U32L) (*(U32L*)pDataTransportBuffer+sizeof(U32L));

      bRet = TII_ReadFrame(TEDSLength, pDataTransportBuffer);
   }
      // ELSE There is no 'End User Application Specific' TEDS implemented: 
      //      return a 'field-length' (4 bytes long) containing '0' to 
      //      indicate that there will be no data-fields to follow.
   else
   {
      U32L NullTEDS = ZERO_LENGTH_FIELD;
      bRet = TII_ReadFrame( 4, (unsigned char*) &NullTEDS );
   }

   return bRet;
}

boolean DAT_ReadXdcrData(unsigned char ucChNum)
{
   boolean       bRet = FALSE;
   unsigned char ucChanCount=0, ucChan=0;
   XDCR_DATA     Data;

      // If ucChNum==CHANNEL_ZERO, read data for ALL channels (starting at 
      //  Channel 1)...
      //
   if(ucChNum==CHANNEL_ZERO) { ucChanCount = NUM_CHANNELS; ucChan = 1; }
   else                      { ucChanCount = 1; ucChan = ucChNum; }

      // Iterate at least once, for the selected channel (if it's not Ch_0).
      // If it is Ch_0, then iterate for ALL the channels.
      //
   while(ucChanCount--)
   {
      bRet=STIM_GetXdcrData(ucChan, (unsigned char*)&Data);

      if(bRet) 
      {
         unsigned char DataLength;

         if(ucChan==1)        DataLength = CH1_DATA_MODEL_LENGTH;
         else if(ucChan==2)   DataLength = CH2_DATA_MODEL_LENGTH;
            //
            // [NOTE] Fill in here for all implemented channels...
            //
         else                 DataLength = 0;
         
         bRet=TII_ReadFrame(DataLength, (unsigned char*)&Data);
      }
         // if transport was aborted, exit FALSE now...
         //
      else break;

      ucChan++;
   }

   return bRet;
}

boolean DAT_ReadStatus(U8E eReg, unsigned char ucChNum)
{
   STATUS_REGISTER StatusReg = 0;
   boolean         bRet = FALSE;
   STATUS_REGISTER StdImplementedMask = (ucChNum==CHANNEL_ZERO)
                                       ?STD_CH_ZERO
                                       :STD_CH_NONZERO;
   STATUS_REGISTER AuxImplementedMask = (ucChNum==CHANNEL_ZERO)
                                       ?AUX_CH_ZERO
                                       :AUX_IMPLEMENTED;

   switch(eReg)
   {
      case(STANDARD):   
               StatusReg = STA_GetStdStatus(ucChNum);
               bRet = TII_ReadFrame( sizeof(STATUS_REGISTER), 
                                    (unsigned char*)&StatusReg);
               if(bRet) STA_ClearStdStatus(ucChNum, STD_SERVICE_REQUEST |
                                                    STD_TRIGGER_ACK     |
                                                    STD_RESET_COMPLETE  |
                                                    STD_INVALID_COMMAND |
                                                    StdImplementedMask  );
         break;
      case(AUXILIARY):
               StatusReg = STA_GetAuxStatus(ucChNum);
               bRet = TII_ReadFrame( sizeof(STATUS_REGISTER), 
                                     (unsigned char*)&StatusReg);
               if(bRet) STA_ClearAuxStatus(ucChNum, AUX_CHANNEL_OUT_OF_RANGE |
                                                    AuxImplementedMask  );
         break;
      default: bRet = FALSE;
   }
   return bRet;
}

boolean DAT_ReadTrigChAddr(void)
{
   unsigned char ucTrigChAddr;

   ucTrigChAddr = TRIG_GetTrigChanAddr();

   return TII_ReadFrame( 1, &ucTrigChAddr);
}

boolean DAT_ReadIntMask(U8E eReg, unsigned char ucChNum)
{
   INTERRUPT_MASK iMask;

   iMask = INT_GetInterruptMask(eReg, ucChNum);

   return TII_ReadFrame(sizeof(INTERRUPT_MASK), (unsigned char*)&iMask);
}

// pauconapr99: this is fine for the current implementation: it's not a generic
//              way to do this though, it'll need modification.
boolean DAT_ReadSTIMVersion(void)
{
   U32L Ver;

   STIM_GetVersion((unsigned char *)&Ver);

   return TII_ReadFrame(sizeof(U32L), (unsigned char *)&Ver);
}

boolean DAT_WriteXdcrData(unsigned char ucChNum)
{
   boolean       bRet = FALSE;
   unsigned char ucChanCount=0;
   unsigned char ucChan=0;

      // If ucChNum==CHANNEL_ZERO, write data for ALL channels (starting at 
      //  Channel 1)...
      //
   if(ucChNum==CHANNEL_ZERO) { ucChanCount = NUM_CHANNELS; ucChan = 1; }
   else                      { ucChanCount = 1; ucChan = ucChNum; }

      // Iterate at least once, for the selected channel (if it's not Ch_0).
      // If it is Ch_0, then iterate for ALL the channels.
      //
   while(ucChanCount--)
   {
      XDCR_DATA     Data;
      unsigned char DataLength;

      if(ucChan==1)        DataLength = CH1_DATA_MODEL_LENGTH;
      else if(ucChan==2)   DataLength = CH2_DATA_MODEL_LENGTH;
         //
         // [NOTE] Fill in here for all implemented channels...
         //
      else                 DataLength = 0;
         
         // Receive 'DataLength' bytes from transport and put them into 'Data'.
         //
      bRet = TII_WriteFrame(DataLength, (unsigned char*)&Data);

         // Write the values just received into the STIM Xdcr Data field.
         //
      if(bRet) STIM_SetXdcrData(ucChan, (unsigned char*)&Data);

         // If transport was aborted, exit FALSE now...
         //
      else     break;

      ucChan++;
   }

   return bRet;
}

boolean DAT_WriteCtrlCmmd(unsigned char ucChNum)
{
   boolean      bRet;
   unsigned int CtrlCmmd;

   bRet = TII_WriteFrame( 2, (unsigned char*)&CtrlCmmd);

   if(bRet)
   {
      switch(CtrlCmmd){
         case(0):        CTRL_NoOp();              break;
         case(1): bRet = CTRL_Reset(ucChNum);      break;
         case(2): bRet = CTRL_SelfTest(ucChNum);   break;
         case(3): bRet = CTRL_Calibrate(ucChNum);  break;
         case(4): bRet = CTRL_Zero(ucChNum);       break;
         default:                                  break;
      }
   }
   return bRet;  
}

boolean DAT_WriteTrigChAddr(void)
{
   boolean       bRet=TRUE;
   unsigned char ucChAddr;

   bRet = TII_WriteFrame( 1, (unsigned char*)&ucChAddr );

   if(bRet) TRIG_SetTrigChanAddr(ucChAddr);

   return (bRet);
}

boolean DAT_WriteIntMask(U8E eMask, unsigned char ucChan)
{
   boolean        bRet=TRUE;
   INTERRUPT_MASK iMask;

   bRet = TII_WriteFrame( 2, (unsigned char*)&iMask );

   if(bRet) INT_SetInterruptMask( eMask, ucChan, iMask );

   return (bRet);
}

boolean DAT_WriteCalibTEDS(void)
{
   // not implemented...
   return FALSE;
}

boolean DAT_WriteCalibIdTEDS(void)
{
   // not implemented...
   return FALSE;
}

boolean DAT_WriteEndUserAppTEDS(void)
{
   // not implemented...
   return FALSE;
}


// Control Functions:
// -----------------

void CTRL_NoOp(void)
{
   return;  
}

boolean CTRL_Reset(unsigned char ucChannel)
{
   boolean bRet = FALSE;
   CHANNEL *pCh = STIM_GetChHandle(ucChannel);

      // a CHANNEL_ZERO reset => all the channels on the STIM must be reset
      //
   if(ucChannel == CHANNEL_ZERO)
   {  
      unsigned char count=NUM_CHANNELS+1;

         // handle CHANNEL_ZERO the same way as all other channels
         //
      while(count--)
      {
         pCh = STIM_GetChHandle(count);
         pCh->StdStatus = 0;  
         pCh->AuxStatus = 0;
         pCh->Data.ChData = 0;
      }
      STIM_SetResetFlag(ucChannel);
   }
   else
   {
      pCh->StdStatus = 0;  
      pCh->AuxStatus = 0;
      pCh->Data.ChData = 0;
   }

   return bRet;
}

boolean CTRL_SelfTest(unsigned char ucChannel)
{
   // this function is not implemented
   return ucChannel;
}

boolean CTRL_Calibrate(unsigned char ucChannel)
{
   // this function is not implemented
   return ucChannel;
}

boolean CTRL_Zero(unsigned char ucChannel)
{
   // this function is not implemented
   return ucChannel;
}



// Interrupt Functions:
// -------------------

INTERRUPT_MASK INT_GetInterruptMask(U8C eFlag, unsigned char ucCh)
{
   if(eFlag==STANDARD)        return STIM_GetChHandle(ucCh)->StdIntMask;
   else if(eFlag==AUXILIARY)  return STIM_GetChHandle(ucCh)->AuxIntMask;
   else                       return NULL;
}

boolean INT_SetInterruptMask(U8C eFlag, unsigned char ucCh, 
                             INTERRUPT_MASK Mask)
{
   boolean bRet=FALSE;

   if(eFlag==STANDARD)
   {  
         // make sure the 'service request' bit is set...
         //
      Mask |= STD_SERVICE_REQUEST;
      STIM_GetChHandle(ucCh)->StdIntMask = Mask;
      bRet = TRUE;
   }
   else if(eFlag==AUXILIARY)
   {
      STIM_GetChHandle(ucCh)->AuxIntMask = Mask;
      bRet = TRUE;
   }

   return bRet;
}

boolean INT_ServiceRequest(void)
{
   unsigned char count;
   boolean bServiceReqd = FALSE;

   STD_STATUS_REGISTER  StdChannelsORed=0;
   AUX_STATUS_REGISTER  AuxChannelsORed=0;
   STATUS_REGISTER      Std, Aux;

   CHANNEL *pChZero = STIM_GetChHandle(CHANNEL_ZERO);
   CHANNEL *pCh     = NULL;

      // Set the 'service request' bits for each channel (excluding 
      // CHANNEL_ZERO) if a 'status' or 'auxiliary' service bit has been set.
      //
   for(count=NUM_CHANNELS; count>CHANNEL_ZERO; count--)
   {
      pCh = STIM_GetChHandle(count);

      if( Std = (pCh->StdStatus & pCh->StdIntMask) )
      {
         STA_SetStatusBit(STANDARD, STD_SERVICE_REQUEST, count);
         bServiceReqd=TRUE;
         StdChannelsORed |= Std;
      }
      if( Aux = (pCh->AuxStatus & pCh->AuxIntMask) )
      {
         STA_SetStatusBit(STANDARD, STD_SERVICE_REQUEST, count);
         bServiceReqd=TRUE;
         AuxChannelsORed |= Aux;
      }
   }
   
      // Now set the CHANNEL_ZERO status bits that need setting
      //
   pChZero->AuxStatus = AuxChannelsORed & (AUX_CHANNEL_OUT_OF_RANGE |
                                           AUX_CHANNEL_CONSUMABLES_EXHAUSTED |
                                           AUX_FAILED_CALIBRATION |
                                           AUX_CHANNEL_BUSY );

   pChZero->StdStatus = (StdChannelsORed & STD_AUXILIARY_STATUS_AVAILABLE) |
                        (pChZero->StdStatus & ( STD_TRIGGER_ACK     |
                                                STD_RESET_COMPLETE  |
                                                STD_INVALID_COMMAND |
                                                STD_OPERATIONAL ) );

   if( pChZero->StdStatus & pChZero->StdIntMask ) 
   {
      STA_SetStatusBit(STANDARD, STD_SERVICE_REQUEST, CHANNEL_ZERO);
      bServiceReqd=TRUE;
   }
   if( pChZero->AuxStatus & pChZero->AuxIntMask ) 
   {
      STA_SetStatusBit(STANDARD, STD_SERVICE_REQUEST, CHANNEL_ZERO);
      bServiceReqd=TRUE;
   }

   INT_ToggleInterrupt(bServiceReqd);

   return bServiceReqd;
}

void INT_ToggleInterrupt(boolean bSet)
{
   if(bSet) TII_AssertNINT();
   else     TII_NegateNINT();
}


// Status Functions:
// ----------------

boolean STA_SetStatusBit(U8E eReg, unsigned int uiBit, unsigned char ucChannel)
{
   boolean bRet=TRUE;

   if(uiBit)
   {
      CHANNEL *pCh = STIM_GetChHandle(ucChannel);

      if(eReg==STANDARD)
         pCh->StdStatus |= (uiBit&STD_IMPLEMENTED);

      else if(eReg==AUXILIARY)
      {
         pCh->AuxStatus |= (uiBit&AUX_IMPLEMENTED);
         pCh->StdStatus |= STD_AUXILIARY_STATUS_AVAILABLE;
      }

      else bRet=FALSE;
   }
   else bRet=FALSE;

   return bRet;   
}

boolean STA_ClearStatusBit(U8E eReg,unsigned int uiBit,unsigned char ucChannel)
{
   boolean bRet=TRUE;

   if(uiBit)
   {
      CHANNEL *pCh = STIM_GetChHandle(ucChannel);

      if(eReg==STANDARD)
         pCh->StdStatus &= ~(uiBit&STD_IMPLEMENTED);

      else if(eReg==AUXILIARY)
      {
         pCh->AuxStatus &= ~(uiBit&AUX_IMPLEMENTED);
         if(pCh->AuxStatus == 0)
            pCh->StdStatus &= ~STD_AUXILIARY_STATUS_AVAILABLE;
      }

      else bRet=FALSE;
   }
   else bRet=FALSE;

   return bRet;   
}

STATUS_REGISTER STA_GetStdStatus(unsigned char ucChannel)
{
      // Get a handle on the channel, then read it's standard status register
      //
   CHANNEL *pCh           = STIM_GetChHandle(ucChannel);
   STATUS_REGISTER  StReg = pCh->StdStatus;

      // Clear all of the Implemented bits from the status register:
      //
   if( STA_ClearStatusBit(STANDARD, STD_IMPLEMENTED, ucChannel) )
      return StReg;
   else
      return NULL;
}

STATUS_REGISTER STA_GetAuxStatus(unsigned char ucChannel)
{
      // Get a handle on the channel, then read its auxiliary status register
      //
   CHANNEL *pCh          = STIM_GetChHandle(ucChannel);
   STATUS_REGISTER StReg = pCh->AuxStatus;

      // Clear all of the Implemented bits from the status register:
      //
   if( STA_ClearStatusBit(AUXILIARY, AUX_IMPLEMENTED, ucChannel) )
      return StReg;
   else
      return NULL;
}

boolean STA_ClearStdStatus(unsigned char ucChannel, STATUS_REGISTER StReg)
{
   CHANNEL *pCh               = STIM_GetChHandle(ucChannel);
   STATUS_REGISTER NegatedReg = ~StReg;

   if(StReg) pCh->StdStatus &= NegatedReg;

   return ((pCh->StdStatus & StReg)==0); 
}

boolean STA_ClearAuxStatus(unsigned char ucChannel, STATUS_REGISTER StReg)
{
   CHANNEL *pCh                = STIM_GetChHandle(ucChannel);
   STATUS_REGISTER  NegatedReg = ~StReg;

   if(StReg)             pCh->AuxStatus &= NegatedReg;
   if(pCh->AuxStatus==0) pCh->StdStatus &= ~STD_AUXILIARY_STATUS_AVAILABLE;

   return ((pCh->AuxStatus & StReg)==0); 
}



// Triggering Functions:
// --------------------

boolean TRIG_SetTrigChanAddr(unsigned char ucChAddr)
{
   mucTrigChanAddr = ucChAddr;
   return TRUE;
}

unsigned char TRIG_GetTrigChanAddr(void)
{
   return mucTrigChanAddr;
}

void TRIG_AckTrigger(void)
{
      // assert the NACK line
      //
   TII_AssertNACK();

      // set the 'trigger acknowledged' bit in the channel's status register
      //
   STA_SetStatusBit(STANDARD, STD_TRIGGER_ACK, mucTrigChanAddr);
}

void TRIG_RemoveTrigAck(void)
{
   TII_NegateNACK();
}

boolean TRIG_PollTrigger(void)
{
      // return TRUE if asserted, FALSE if not asserted
      //
   return TII_DetectNTRIG();
}

