/* 
   IEEE 1451.2 Implementation and Authoring Toolset
   ------------------------------------------------

   Target Platform: Analog Devices Aduc812, and Evaluation Board.

   This software is a complete working version that implements 2 channels
   on the IEEE 1451.2 standard.
   - The first channel is a simple temperature sensor (the AD590).
   - The second channel is a simple actuator (a fan with two states: on or off)

   Written by:    Paul Conway
                  PEI Technologies Ltd.
                  Foundation Building
                  National Technological Park
                  Limerick
                  Ireland
                  http://www.ul.ie/~pei

   For:           Analog Devices

   Version:       1.01
*/

#include <aduc812.h>
#include "datatype.h"
#include "function.h"
#include "stim.h"
#include "tii.h"
#include "teds.h"

/*
   Note:
   throughout the module definitions, the following coding practice
   is in use:
   - The module a function belongs in shall be indicated by a 2 - 4 
     letter prefix to the function name.
   - A function name containing the word 'read' indicates that this
     function sends data from the STIM to an NCAP over the TII.
   - A function name containing the word 'write' indicates that this
     function receives data from an NCAP and stores it inside the STIM.
   - A function name containing the word 'set' indicates that this 
     function will set a value internal to the STIM.
   - A function name containing the word 'get' indicates that this
     function gets a value that is already internal to the STIM.
*/

// Channel Sensor / Actuator specific definitions to the 
// current implementation go here.
// -----------------------------------------------------
//
// Define the bit that the Actuator (i.e. the Fan) on channel 2 
// is controlled by.
//
sbit FAN_CH2  = P2 ^ 7;


// Module Level Variables:
// ----------------------

   // The 'reset flag' indicates if a 'reset' was requested, and if so
   // for which channel. It defaults to 'invalid channel' to indicate 
   // that there has not been a reset requested for any channel.
   //
static U8E  meResetFlag = INVALID_CHANNEL;

   // Declare the Channels that will be in use. The channels are accessed
   // via a 'CHANNEL' array. The channel located at position 0 in the 
   // array will represent CHANNEL_ZERO, the channel located at position
   // 1 will represent CHANNEL 1, etc.
   //
   // note: there will be one CHANNEL for each XDCR, and one for CH_ZERO,
   //       hence the declaration for 'NUM_CHANNELS+1'
   //  
CHANNEL maChannelData[NUM_CHANNELS+1];


// STIM Functions:
// ---------------

CHANNEL* STIM_GetChHandle(unsigned char ucChNum)
{
   if(ucChNum<=NUM_CHANNELS)  return &maChannelData[ucChNum];
   else                       return NULL;
}

boolean STIM_GetXdcrData(unsigned char ucChNum, unsigned char* pBuf)
{
   boolean ret=TRUE;

   if(pBuf)
   {  
      switch(ucChNum) 
      {
         case 1:     *(unsigned int*)pBuf = maChannelData[ucChNum].Data.Ch1;  
            break;
         case 2:                    *pBuf = maChannelData[ucChNum].Data.Ch2;  
            break;
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Note: This is where data for any other channels will be returned.
            Replace 'X' with the channel number.
            The data-type will determine how the data is returned.
      
         case X:                    *pBuf = maChannelData[ucChNum].Data.ChX;  
            break;
   */
         default:    ret=FALSE;
      }
   }
   else 
      ret=FALSE;

   return ret;
}

boolean STIM_SetXdcrData(unsigned char ucChNum, unsigned char *pBuf)
{
   boolean ret=TRUE;

   if(pBuf) 
   {  
      switch(ucChNum) 
      {
         case 1:  maChannelData[ucChNum].Data.Ch1 = *pBuf;
                  maChannelData[ucChNum].Data.Ch1 <<= 8;
                  maChannelData[ucChNum].Data.Ch1 |= *(pBuf+1);
               break;
         case 2:  maChannelData[ucChNum].Data.Ch2 = *pBuf;  
               break;
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Note: This is where data for any other channels will be set.
            The data-type will determine how the data is set.
   */      
         default: ret=FALSE;
      }
   }
   else    ret=FALSE;

   return ret;
}

void STIM_GetVersion(char* pVersion)
{
      // let's keep it simple for now...
      //
   U32L Version = 0x00000000;
   
   if(pVersion)   *pVersion = Version;
}

void STIM_SoftReset(void)
{
   unsigned char count;

      // reset all channels
      //
   for(count=0; count<(NUM_CHANNELS+1); count++)
   {
         // Reset the standard and auxiliary status registers. 
         // Reset the 'transducer-data' field.
         //
      maChannelData[count].StdStatus = 0;
      maChannelData[count].AuxStatus = 0;
      maChannelData[count].Data.ChData = 0;

         // set the 'reset' and 'operational' bits in standard status register
         //
      maChannelData[count].StdStatus |= STD_RESET_COMPLETE;
      maChannelData[count].StdStatus |= STD_OPERATIONAL;
   }
}

void STIM_InitialReset(void)
{
   unsigned char count;

      // do the standard 'soft' reset for all channels
      //
   STIM_SoftReset();

   for(count=0; count<(NUM_CHANNELS+1); count++)
   {
         // reset standard and auxiliary interrupt registers. 
         //
      maChannelData[count].StdIntMask = 0xFFFF;
      maChannelData[count].AuxIntMask = 0x0000;
   }
}

void STIM_ChannelReset(unsigned char ucChNum)
{
   if( ucChNum>0 && ucChNum<(NUM_CHANNELS+1) )
   {
      maChannelData[ucChNum].StdStatus = 0;
      maChannelData[ucChNum].AuxStatus = 0;
      maChannelData[ucChNum].Data.ChData = 0;

         // set the 'reset' and 'operational' bits in standard status register
         //
      maChannelData[ucChNum].StdStatus |= STD_RESET_COMPLETE;
      maChannelData[ucChNum].StdStatus |= STD_OPERATIONAL;

         // note: don't change the 'channel data length' field.
   }
}

void STIM_SetResetFlag(U8E eFlag)
{
   meResetFlag = eFlag;
}

U8E STIM_GetResetFlag(void)
{
   return meResetFlag;
}

// Initialize the channels specific to this implementations:
// --------------------------------------------------------
//
void STIM_InitCh1(void)
{
   // Initialise the Sensor (AD590):
   // -----------------------------
      // Set up Port 1.0 as an analog input port (i.e. Sensor Input on ADC0)
      //    - this is done by writing 1 to that SFR bit.
      //
   P1 = P1 | 0x01;

   ADCCON1 = 0x6C;   // ADCCON1  01101100
                     // SFR explanatory comments...
                     //   MD1 MD0
                     //     0   1 => ADC normal mode
                     //   CK1 CK0
                     //     1   0 => Clock divide ratio = 4
                     //   AQ1 AQ0
                     //     1   1 => time for acquisition = 8 ADC clks
                     //   T2C
                     //     0 => timer 2 overflow not used as conv start bit
                     //   EXC
                     //     0 => ext pin 23 not used as conv start bit
   ADCCON3 = 0x00;   // ADCCON3  00000000
                     // SFR explanatory comments...
                     //   BUSY X X X CTYP CAL1 CAL0 CALST
                     //   BUSY = 0 => read only. set during conversion.
                     //   CTYP = 0 => Internal calibration
                     //   CAL1 CAL0
                     //      0    0 => 'full' calibration?? - not used now.
                     //   CALST = 0 => calibration not initiated

   ADCCON2 = 0x00;   //  Don't enable any data conversion mode yet:
                     //  SFR explanatory comments...
                     //   ADCI=0 => interrupt bit. will be set at end of cycle.
                     //   DMA=0 => NOT pre-configured DMA mode
                     //   CCONV=0 => NOT continuous conversion mode
                     //   SCONV=0 => DON'T initiate single conversion mode YET!
                     //   4 LSBs=0 => ch 0 selected

}

void STIM_InitCh2(void)
{
   // Initialise the Actuator:
   // -----------------------
      // Set up Port 2.7 as a digital output (i.e. Actuator Output)
      //    - this is done by writing 0 to that SFR bit.
      //
   FAN_CH2 = 0;
}

unsigned int STIM_GetCh1Sample(void)
{
   unsigned int  uiSample1=0;
   unsigned char ucSample2=0;

      // Set the 'single sample' bit in ADCCON2 to initiate a 'single 
      // conversion cycle' on ADC channel 0
      //
   ADCCON2 = 0x00;   // ADC channel 0
   SCONV = 1;        // ...single conversion mode
   
      // Wait for the ADC interrupt (i.e. sample acquired) bit ...
      //
   while( !ADCI )
      ;

      // A sample has been acquired - read it out of the data SFR
      //
   uiSample1 = ADCDATAH;
   ucSample2 = ADCDATAL;
   
   uiSample1 <<= 8;
   uiSample1 = uiSample1 | ucSample2;

   SCONV = 0;
   ADCI = 0;

      // now return the 12-bit sample
      //
   return (uiSample1 & 0x0FFF);
}

void STIM_SetCh2State(boolean bState)
{
   FAN_CH2 = (bState) ? 1 : 0 ;
}


//
// --------------------
// The 'main' function.
// This function encapsulates the 'STIM kernel' - it is this function
// that essentially controls the operation of the 1451.2 implementation
// --------------------------------------------------------------------
//
void main(void)
{
   //
   // Power On Initialisation Routine:
   // -------------------------------
      // Setup the channel information in RAM. Set default values.
      //
   STIM_InitialReset();
   STIM_InitCh1();
   STIM_InitCh2();

      // Write the TEDS information into the TEDS flash storage area.
      //
   TEDS_InitTEDSFlash();
   TEDS_SetupMetaTEDS();
   TEDS_SetupCh1TEDS();
   TEDS_SetupCh2TEDS();

      // Setup the Interrupt Mask values for this STIM:
      //
      // Set the Standard Interrupt Masks for CHANNEL_ZERO, Ch1 and Ch2...
      //
   INT_SetInterruptMask(STANDARD, CHANNEL_ZERO, STD_SERVICE_REQUEST |
                                                STD_TRIGGER_ACK |
                                                STD_RESET_COMPLETE |
                                                STD_INVALID_COMMAND | 
                                                STD_AUXILIARY_STATUS_AVAILABLE);
   INT_SetInterruptMask(STANDARD, 1, STD_SERVICE_REQUEST |
                                     STD_TRIGGER_ACK |
                                     STD_RESET_COMPLETE |
                                     STD_AUXILIARY_STATUS_AVAILABLE );
   INT_SetInterruptMask(STANDARD, 2, STD_SERVICE_REQUEST    |
                                     STD_TRIGGER_ACK        |
                                     STD_RESET_COMPLETE     |
                                     STD_AUXILIARY_STATUS_AVAILABLE);

      // Set the Auxiliary Interrupt Masks for CHANNEL_ZERO, Ch1 and Ch2...
      // NOTE: the auxiliary interrupt mask in this STIM is 0 for all channels.
      //
   INT_SetInterruptMask(AUXILIARY, CHANNEL_ZERO, 0x0000);
   INT_SetInterruptMask(AUXILIARY, 1,            0x0000);
   INT_SetInterruptMask(AUXILIARY, 2,            0x0000);

      // Setup the TII data-transport interface.
      //
   TII_Initialise();


   // Start Loop:
   // ----------

   do 
   {
      // Has a Reset been requested? (a 'control' request)
      // --------------------------
      {
         U8E eReset = STIM_GetResetFlag();

            // Perform a 'soft' reset - this won't affect interrupt masks
            //
         if(eReset == CHANNEL_ZERO)          STIM_SoftReset();
         else if(eReset != INVALID_CHANNEL)  STIM_ChannelReset(eReset);

         STIM_SetResetFlag(INVALID_CHANNEL);
      }


      // Is the Data Transport Active?
      // ----------------------------

      if( DAT_TransportActive() )
      {
         unsigned char FnAddr=0, ChAddr=0;
         
         TII_SetAbortStatus(FALSE);

            // Data Transport begins by NCAP sending an Address to the STIM:
            //
         if(!TII_GetAbortStatus())   FnAddr = TII_WriteByte();
         if(!TII_GetAbortStatus())   ChAddr = TII_WriteByte();
         
            // The Data Transport has not aborted: continue on to the
            // transport protocol...
            //
         if(!TII_GetAbortStatus())
         {
               // ucRdWr will be FN_READ for a 'read' function, FN_WRITE for 
               // a 'write' function.
               //
            unsigned char ucRdWr = FnAddr & READ_FN;
            FnAddr &= 0x7F;

               // The Transport has requested a READ function:
               //
            if(ucRdWr == READ_FN)
            {
               switch(FnAddr)
               {
                  case(TRANSDUCER_DATA):     DAT_ReadXdcrData(ChAddr);
                  break;

                  case(STANDARD_STATUS):     DAT_ReadStatus(STANDARD, ChAddr);
                  break;

                  case(TRIGGERED_CHANNEL_ADDRESS): 
                                             DAT_ReadTrigChAddr();
                  break;

                  case(AUXILIARY_STATUS):    DAT_ReadStatus(AUXILIARY, ChAddr);
                  break;

                  case(STANDARD_INTERRUPT_MASK):
                                             DAT_ReadIntMask(STANDARD, ChAddr);
                  break;

                  case(AUXILIARY_INTERRUPT_MASK):
                                             DAT_ReadIntMask(AUXILIARY, ChAddr);
                  break;                         

                  case(STIM_VERSION):        DAT_ReadSTIMVersion();
                  break;                         

                  case(META_TEDS | CHANNEL_TEDS):
                        if(ChAddr==CHANNEL_ZERO) DAT_ReadMetaTEDS();
                        else                     DAT_ReadChannelTEDS(ChAddr);
                  break;

                  case(META_ID_TEDS | CHANNEL_ID_TEDS):
                        if(ChAddr==CHANNEL_ZERO) DAT_ReadMetaIdTEDS();
                        else                     DAT_ReadChannelIdTEDS(ChAddr);
                  break;

                  case(CALIBRATION_TEDS):        DAT_ReadCalibrationTEDS();
                  break;

                  case(CALIBRATION_ID_TEDS):     DAT_ReadCalibrationIdTEDS();
                  break;

                  case(USERS_APP_SPECIFIC_TEDS): DAT_ReadEndUserAppTEDS();
                  break;

                  default:    
                  STA_SetStatusBit(STANDARD, STD_INVALID_COMMAND, CHANNEL_ZERO);
                  break;
               }
            }
               // The Transport has requested a WRITE function:
               //
            else
            {
               switch(FnAddr)
               {
                  case(TRANSDUCER_DATA):  DAT_WriteXdcrData(ChAddr);
                  break;

                  case(CONTROL_COMMAND):  DAT_WriteCtrlCmmd(ChAddr);
                  break;

                  case(TRIGGERED_CHANNEL_ADDRESS): 
                                          DAT_WriteTrigChAddr();
                  break;

                  case(STANDARD_INTERRUPT_MASK):   
                                          DAT_WriteIntMask(STANDARD, ChAddr);
                  break;

                  case(AUXILIARY_INTERRUPT_MASK):   
                                          DAT_WriteIntMask(AUXILIARY, ChAddr);
                  break;

                  case(CALIBRATION_TEDS): DAT_WriteCalibTEDS();
                  break;

                  case(CALIBRATION_ID_TEDS):
                                          DAT_WriteCalibIdTEDS();
                  break;

                  case(USERS_APP_SPECIFIC_TEDS):
                                          DAT_WriteEndUserAppTEDS();
                  break;

                  default:
                  STA_SetStatusBit(STANDARD, STD_INVALID_COMMAND, CHANNEL_ZERO);
                  break;
               }
            }
         }

            // Either transport was Aborted, or Transport is finished... 
            // return NACK to it's idle/passive state.
            //
         TII_NegateNACK();
      }

      // Is Trigger Asserted?
      // -------------------

      if(TRIG_PollTrigger())
      {  
         unsigned char ucChAddr = TRIG_GetTrigChanAddr();

         switch( ucChAddr )
         {
            unsigned int  Ch1Sample;
            unsigned char Ch2Data;

            case(CHANNEL_ZERO):
                                 // SENSOR channel, AD590:
            case(1):             // 1. Acquire a data sample
                                 //
                     Ch1Sample=STIM_GetCh1Sample();

                                 // 2. Acknowledge the trigger at acquire time
                                 //
                     TRIG_AckTrigger();

                                 // 3. Store sample into channel data area
                                 //
                     STIM_SetXdcrData( 1, (unsigned char*)&Ch1Sample );

                                 // 4. Remove Acknowledge when trigger has
                                 //    been removed.
                     while(TII_DetectNTRIG())
                        ;
                     TRIG_RemoveTrigAck();
                     
                                 // 5. Acknowledge removed - go back to 
                                 //    'quiescent' state

            if(ucChAddr != CHANNEL_ZERO) 
            break;

            case(2):             // ACTUATOR channel, Fan:
                                 // 1. Assume there is valid data to be written
                                 //
                     STIM_GetXdcrData( 2, &Ch2Data );

                                 //  2. Write the 'data set' to the actuator
                                 //
                     STIM_SetCh2State( Ch2Data );

                                 // 3. Acknowledge trigger - we have just 
                                 //    changed the actuator state
                     TRIG_AckTrigger();

                                 // 4. Wait for the trigger to be removed, 
                                 //    then remove the acknowledge.
                     while(TII_DetectNTRIG())
                        ;
                     TRIG_RemoveTrigAck();

                                 // 5. Acknowledge removed - go back to 
                                 //    'quiescent' state

            if(ucChAddr != CHANNEL_ZERO) 
            break;

            default:
                     // The channel requested does not exist.
                     // Go back to the 'quiescent' state.
            break;
         }
      }

      // Does STIM need to be serviced?
      // -----------------------------
         // If there are requests for service, service them.
         //
      INT_ServiceRequest();
                               
   } while(1);
}

